/*
 *  Licensed Materials - Property of IBM Corp.
 *  5648-C60 (c) Copyright IBM Corp. 1999
 *
 *  Point to Point Sample02
 *
 *  Demonstrates:
 *    how to use a MessageListener for asynchronous message delivery
 *    how to use message selectors to filter the input stream
 *
 * The program creates two QueueReceivers on the same queue, each with
 * a MessageListener.  One QueueReceiver has a message selector for the
 * user defined property colour=red, the other has a selector for
 * colour=blue. A QueueSender is used to send four messages to the
 * queue, two with colour=red, two with colour=blue. These should be
 * received and displayed by the appropriate MessageListener instance.
 *
 * The program assumes that the administered objects used for the
 * installation verification test program can be retrieved from a JNDI
 * namespace. Edit the 'icf' and 'url' strings below to match your
 * installation.
 *
 *  Usage:
 *
 * java PTPSample02 
 */

import javax.jms.*;

// Imports for JNDI
import javax.naming.*;
import javax.naming.directory.*;

// Used for JNDI initialisation
import java.util.Hashtable;

public class PTPSample02
{

  // The following can be modified if you want to experiment with
  // using other administered objects with different content.
  public static final String  qcfLookup    = "ivtQCF";
  public static final String  qLookup      = "ivtQ";

  // edit these to match your environment
  public static final String icf = "com.sun.jndi.ldap.LdapCtxFactory";
  public static final String url = "ldap://sirius/o=ibm_us,c=us";

  /** entry point when called from command line. */
  public static void main(String[] args)
  {
    QueueSession session = null;
    QueueConnection connection = null;
    QueueConnectionFactory factory = null;

    System.out.println("PTPSample02 - simple example of using asynchronous message delivery.");

    try {

      System.out.println("Getting administered objects from JNDI");

      Context ctx = initJNDI(icf, url);
      factory = getConnectionFactoryFromJNDI(ctx, qcfLookup);
      Queue Q1 = getQueueFromJNDI(ctx, qLookup);

      // Create a connection
      System.out.println("creating connection");
      connection = factory.createQueueConnection();

      // Create a session
      System.out.println("creating session");
      session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

      // obtain queue
      System.out.println("creating JMS Queue");
      Queue q1 = session.createQueue("SYSTEM.DEFAULT.LOCAL.QUEUE");

      // Create QueueReceiver with a message selector for 'red' messages
      System.out.println("creating queue receiver");
      QueueReceiver receiver1 = session.createReceiver(q1, "colour='red'");

      // register a listener
      System.out.println("registering message listener 1 for red messages");
      PTPSample02MListener listener1 = new PTPSample02MListener(1);
      receiver1.setMessageListener(listener1);

      // Create another QueueReceiver for 'blue' messages
      System.out.println("creating queue receiver");
      QueueReceiver receiver2 = session.createReceiver(q1, "colour='blue'");

      // register a listener
      System.out.println("registering message listener 2 for blue messages");
      PTPSample02MListener listener2 = new PTPSample02MListener(2);
      receiver2.setMessageListener(listener2);

      // Create a QueueSender
      System.out.println("creating a queue sender");
      QueueSender queueSender = session.createSender(q1);
      queueSender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

      // Create a message
      TextMessage outMessage = session.createTextMessage();

      for(int i=0; i<4; i++) {
	if (i%2 == 0) {
	  System.out.println("Sending red message "+i+" to the queue");
	  outMessage.setText("this is message "+i);
	  outMessage.setStringProperty("colour", "red");
	} else {
	  System.out.println("Sending blue message "+i+" to the queue");
	  outMessage.setText("this is message "+i);
	  outMessage.setStringProperty("colour", "blue");
	}
	queueSender.send(outMessage);
      }

      // Now that we have stopped sending messages, we can start the
      // connection. This also starts asynchronous receive on
      // the session - it would be illegal to send a message while
      // asynchronous receive was active on the same session.
      System.out.println("starting connection");
      connection.start();

      Thread.sleep(5000);	// give the receivers time to print messages

    } catch (JMSException je) {
      System.err.println("caught "+je);
      Exception e = je.getLinkedException();
      if (e != null) {
        System.err.println("subexception: "+e);
      }

    } catch (Exception e) {
      System.err.println("caught "+e);

    } finally {
      if (session != null) {
        try {
          System.out.println("closing session");
          session.close();
        } catch (JMSException e) {
          System.err.println("caught "+e);
        }
      }
      if (connection != null) {
        try {
          System.out.println("closing connection");
          connection.close();
        } catch (JMSException e) {
          System.err.println("connection.close() threw "+e);
        }
      }

    }

    System.out.println("Finished ===============================\n");
		
  }

  //=========================================================================
  /** create an InitialDirContext for retrieving JNDI objects. */
  static InitialDirContext initJNDI(String icf, String url)
    throws JMSException, Exception
  {
    // the context to be returned from this routine
    InitialDirContext ctx = null;
    
    // hashtable for the JNDI initialisation properties
    Hashtable environment = new Hashtable();

    if( url==null ) {
      // A URL must be supplied, so throw an exception if none is available
      throw new Exception("You must specify the -url providerURL parameter");
    }

    // Firstly, store the name of the 'initial context factory'
    environment.put(Context.INITIAL_CONTEXT_FACTORY, icf);

    // Secondly, specify the location of the JNDI service provider
    environment.put(Context.PROVIDER_URL, url);

    // Finally, ensure that referrals are thrown. This is required
    // as a workaround to a problem encountered with certain comb-
    // inations of JNDI service provider and JNDI classes
    environment.put(Context.REFERRAL, "throw");

    try {
      // Attempt to connect to the JNDI service provider, supply-
      // ing the properties previously supplied
      ctx = new InitialDirContext( environment );

    } catch( Exception e ) {
      // return an exception to the caller
      JMSException je = new JMSException("Cannot create JNDI InitialContext!");
      je.setLinkedException(e);
      throw je;
    }

    return ctx;

  }

  //=========================================================================
  /** Get the named QueueConnectionFactory object from the JNDI namespace.
   */
  static QueueConnectionFactory getConnectionFactoryFromJNDI(Context ctx,
							     String name)
    throws JMSException
  {
    // the factory to be returned
    QueueConnectionFactory factory = null;

    System.out.println( "Retrieving a QueueConnectionFactory from JNDI");

    try {
      // Attempt to retrieve a QueueConnectionFactory from the
      // JNDI namespace
      factory = (QueueConnectionFactory)ctx.lookup( "cn="+name );

    } catch( Exception e ) {
      // If this has failed, try the operation again without the cn prefix
      try {
	factory = (QueueConnectionFactory)ctx.lookup( name );

      } catch( Exception e2 ) {
	// After a second failure pass an exception to the caller
	JMSException je =
	  new JMSException("Unable to retrieve the QCF object from JNDI");
	je.setLinkedException(e2);
	throw je;
      }
    }

    return factory;

  }

  //=========================================================================
  /** Get the named Queue object from the JNDI namespace. */
  static Queue getQueueFromJNDI(Context ctx, String name)
    throws JMSException
  {
    // The Queue to be returned
    Queue ioQueue = null;

    System.out.println("Retrieving a Queue from JNDI");
    try {
      // Attempt to retrieve a Queue from the JNDI namespace
      ioQueue = (Queue)ctx.lookup( "cn="+name );
    } catch( Exception e ) {
      try {
	// If this fails, attempt again without the "cn=" prefix
	ioQueue = (Queue)ctx.lookup( name );

      } catch( Exception e2 ) {
	// After a second failure pass an exception back to the caller
	JMSException je = new JMSException("Unable to retrieve the Queue '"+
					   name+"' from JNDI");
	je.setLinkedException(e2);
	throw je;
      }
    }

    return ioQueue;
  }
}

/*
 * This class provides the message listener.
 */
class PTPSample02MListener implements MessageListener
{
  // an id to allow us to distinguish between different instances
  int id;

  PTPSample02MListener(int id) {
    this.id = id;
  }

  
  /** This method provides the implementation of the MessageListener
      interface. */
  public void onMessage(Message message)
  {
    String body;

    try {
      System.out.println("in onMessage of listener "+id);

      if (message instanceof TextMessage) {
	// display the message contents
	body = ((TextMessage)message).getText();
	if (body != null) {
	  System.out.println("message contained '"+body+"'");
	} else {
	  System.out.println("error: message contained no body");
	}
	// display the message 'colour' property
	String colour = message.getStringProperty("colour");
	if (colour == null) {
	  System.out.println("the message didn't have a colour property!");
	} else {
	  System.out.println("colour="+colour);
	}
      } else {
	System.out.println("error: message was not a TextMessage as expected");
	System.out.println(message);
      }

    } catch (JMSException je) {
      System.out.println("onMessage caught "+je);
      Exception e = je.getLinkedException();
      if (e != null) System.out.println("linked exception: "+e);
      
    } catch (Exception e) {
      System.out.println("onMessage caught "+e);
    }
  }

}
